# aaa_subs.tcl 

######################################################################
# Copyright 2000 Nokia IP, Inc.  All rights reserved.
#
# YOU MAY ONLY USE THE MATERIALS FOUND IN THIS FILE PURSUANT TO THE
# TERMS OF THE EVALUATION LICENSE AGREEMENT FOUND IN "/LICENSE".  ALL
# OTHER USES ARE EXPRESSLY PROHIBITED.
######################################################################
#
# $Id: aaa_subs.tcl,v 1.23 2002/11/11 20:02:06 btreppa Exp $
#

#
# Procedures:
#   prepare_new
#   prepare_delete
#   incrIP
#   check_aaa_bindings
#   acquire_aaa_config
#   restore_defaults
#   pam_type_chgd
#   more_profile_params
#   do_select_delete

#********************************************************************
# Procedure   : prepare_new
# Description :
# Arguments   :
# Returns     :
# See also    :
#*******************************************************************/

proc prepare_new { old } {
    # inverts "new:aaa:" to "aaa:new" for CDK
    regsub "new:aaa" $old "aaa:new" new
    return $new
}

#********************************************************************
# Procedure   : prepare_delete
# Description :
# Arguments   :
# Returns     :
# See also    :
#*******************************************************************/

proc prepare_delete { old } {
    # removes "delete:" for CDK
    regsub "delete:" $old "" new
    return $new
}

#********************************************************************
# Procedure   : incrIP
# Description :
# Arguments   :
# Returns     :
# See also    :
#*******************************************************************/

proc incrIP { addr } {
    set old_ip [split $addr "."]
    set ipFld4 [lindex $old_ip end]
    incr ipFld4
    if { $ipFld4 > 254} {
	set ipFld4 0
    }
    set new_ip [lreplace $old_ip end end $ipFld4]
    return [join $new_ip "."]
}

#********************************************************************
# Procedure   : check_aaa_bindings
# Description :
# Arguments   :
# Returns     :
# See also    :
#*******************************************************************/

proc check_aaa_bindings { } {
    global aaa_current Q_Names Q_Data aaa_debug myDb err_list dbgf set_list
    global myCl is_cluster cid USER

    set error_flag 0
    set upd_cnt 0
    set del_cnt 0
    set add_cnt 0
    set bind_cnt 0
    set ip_addr_cnt 0
    set new_params [list]
    if { $aaa_debug == 2 } {
      puts $dbgf "aaa_current in check_aaa_bindings:"
      puts $dbgf "[ array names aaa_current ]"
    }

    if {[Validate $Q_Data err_list] == 1} {
	set error_flag 1
	return $error_flag
    }

    foreach var $Q_Names {
      incr bind_cnt
      if { $aaa_debug == 3 } {
	puts $dbgf "Binding: $var"
      }
      # items that are in aaa_current() include "new:" and "changed"
      if { [info exists aaa_current($var)] != 0 } {
	if { $aaa_debug == 3 } {
	  puts $dbgf "Found aaa_current entry for $var"
	}
	# first check new values (they now exist and have
	# format "new:<db_var>")
	if { [ regexp {^new:} $var ] != 0 } {
	    # new value is [getVal ...]
  	    # true new variables have bindings with values that have changed
  	    set new_val [getVal $var]
	    if { $new_val != "" } {
  	      set old_val $aaa_current($var)
	      if { $old_val != $new_val } {
		incr add_cnt
		if { $aaa_debug == 3 } {
		  puts $dbgf "user added $var: \"$new_val\" (over \"$old_val\")"
		}
		set aaa_current($var) $new_val
		set var [prepare_new $var]
		if { [string first "_type" $var] != -1 } {
		    # some special handling for *_type
		    # only include if user changed it or
		    # there are other params changed
		    if { [pam_type_chgd $var $new_val] || 
			     [more_profile_params $var $new_params] } {
			if { $aaa_debug == 4 } {
			    puts $dbgf "$var $new_val"
			}
			lappend new_params $var $new_val
		    }
		} else {
		    if { $aaa_debug == 4 } {
			puts $dbgf "$var $new_val"
		    }
		    lappend new_params $var $new_val
		}
	    } ;# $old_val != $new_val
	  } ;# $new_val != ""
	} else {
	  set bind_val [getVal $var]
	  set curr_val $aaa_current($var)
	  if { $aaa_debug == 3 } {
	    puts $dbgf "Compare: Value = $bind_val, Current = $curr_val"
	  }
	  if { $bind_val != $curr_val } {
	    if { $aaa_debug == 3 } {
	      puts  $dbgf "user updated \"$curr_val\" to \"$bind_val\""
	    }
	    incr upd_cnt
	    if { $aaa_debug == 4 } {
		puts $dbgf "$var $bind_val"
	    }
	    lappend set_list $var $bind_val
	  }
	}
      } else {
	if { [ regexp {^delete:} $var ] != 0 } {
	  # format delete check box variable as "delete:<db_var>"
	  set var [prepare_delete $var]
	  set sel [lsearch $Q_Names select:$var:*]
	  if { $sel != -1 } {
	    do_select_delete $sel $var
	  } else {
	    if { $aaa_debug == 3 } {
	        puts  $dbgf "user deleted $var "
	    }
	    incr del_cnt
	    lappend set_list $var ""
	  }
	}
      }
    }

    # submit new parameters
    if {[llength $new_params] > 0} {
        if { $aaa_debug == 4 } {
            puts "new parameters:"
            foreach param $new_params {
                puts $dbgf "$param"
            }
        } 
        set set_list [concat $set_list $new_params]
    }
    # submit changed, deleted parameters
    if {[llength $set_list] > 0} {
	if { $aaa_debug > 0 } {
		puts $dbgf "set_list:"
		foreach {name val} ${set_list} {
		    puts $dbgf "\t$name $val"
		}
	}
        if {$is_cluster} {
            set error_flag \
                [eval "catch {cluster db set $myCl -user $USER ${set_list}} \
                 set_result"]

        } else {
	    set error_flag \
		[eval "catch {libdb set ${myDb} -list ${set_list}} set_result"]
        }
	if { [string length $set_result] > 0 } {
	     # include errors and notifys
	     lappend err_list  $set_result
	     set set_result {}
	}
    }


    if { $aaa_debug == 3 } {
      puts $dbgf "User has updated $upd_cnt variables"
      puts $dbgf "User has deleted $del_cnt variables"
      puts $dbgf "User has added $add_cnt variables"
    }
    if { $aaa_debug > 0 } {
      puts $dbgf "Processed $bind_cnt bindings"
    }

    if { [expr $upd_cnt + $del_cnt + $add_cnt] > 0 } {
      acquire_aaa_config
    }
    if { $error_flag != 0 } {
	set err_list [linsert $err_list 0 \
		"Error: `Commit' failed - values not applied to DataBase"]
        if { $aaa_debug == 3 || $aaa_debug == 6 } {
	  puts $dbgf "err_list: ([llength $err_list] items)"
	  puts $dbgf "[join $err_list \n]"
        }
    }

    if { $aaa_debug == 5 } {
      puts $dbgf "check_aaa_bindings returns $error_flag"
    }
    return $error_flag
}

#********************************************************************
# Procedure   : acquire_aaa_config
# Description :
# Arguments   :
# Returns     :
# See also    :
#*******************************************************************/

proc acquire_aaa_config {} {
  global aaa_current aaa_debug dbgf
  global myDb svc_names
  global auth_sets acct_sets sess_sets 
  global svc_profiles
  global auth_pam_types acct_pam_types sess_pam_types

  set svc_names_in_db [ libdb iterate children $myDb aaa:service ]
  set svc_names [list]
  foreach svc_name $svc_names_in_db {
      set aaa_current(aaa:service:$svc_name) \
				  [libdb get $myDb aaa:service:$svc_name] 
      if { $aaa_current(aaa:service:$svc_name) == "t" } {
	lappend svc_names $svc_name
	set aaa_current(aaa:service:$svc_name:profile) \
			  [libdb get $myDb aaa:service:$svc_name:profile] 
      }
  }

  set svc_profiles_in_db [ libdb iterate children $myDb aaa:profile ]
  set svc_profiles [list]
  foreach svc_profile $svc_profiles_in_db {
      set aaa_current(aaa:profile:$svc_profile) \
				  [libdb get $myDb aaa:profile:$svc_profile] 
      if { $aaa_current(aaa:profile:$svc_profile) == "t" } {
	lappend svc_profiles $svc_profile
	# lists of Auth. Profiles, Acct. Profiles and Session Profiles
	# built in "aaa.tcl"
      }
  }

  set auth_sets_in_db [ libdb iterate children $myDb aaa:auth_profile ]
  set auth_sets [list]
  foreach auth_set $auth_sets_in_db {
      set aaa_current(aaa:auth_profile:$auth_set) \
				[ libdb get $myDb aaa:auth_profile:$auth_set ] 
      if { $aaa_current(aaa:auth_profile:$auth_set) == "t" } {
	lappend auth_sets $auth_set
	set aaa_current(aaa:auth_profile:$auth_set:auth_type) \
			 [libdb get $myDb aaa:auth_profile:$auth_set:auth_type]
	set aaa_current(aaa:auth_profile:$auth_set:auth_control_flag) \
	       [ libdb get $myDb aaa:auth_profile:$auth_set:auth_control_flag ]
	set aaa_current(aaa:auth_profile:$auth_set:file_path) \
	       [ libdb get $myDb aaa:auth_profile:$auth_set:file_path ]
	# in future, handle lists of servers
	set aaa_current(aaa:auth_profile:$auth_set:auth_server) \
	       [ libdb get $myDb aaa:auth_profile:$auth_set:auth_server ]
      }
  }

  set acct_sets_in_db [ libdb iterate children $myDb aaa:acct_profile ]
  set acct_sets [list]
  foreach acct_set $acct_sets_in_db {
      set aaa_current(aaa:acct_profile:$acct_set) \
			 [ libdb get $myDb aaa:acct_profile:$acct_set ] 
      if {  $aaa_current(aaa:acct_profile:$acct_set) == "t"  } {
	lappend acct_sets $acct_set
	set aaa_current(aaa:acct_profile:$acct_set:acct_type) \
	       [ libdb get $myDb aaa:acct_profile:$acct_set:acct_type ]
	set aaa_current(aaa:acct_profile:$acct_set:acct_control_flag) \
	       [ libdb get $myDb aaa:acct_profile:$acct_set:acct_control_flag ]
	set aaa_current(aaa:acct_profile:$acct_set:file_path) \
	       [ libdb get $myDb aaa:acct_profile:$acct_set:file_path ]
	# in future, handle lists of servers
	set aaa_current(aaa:acct_profile:$acct_set:acct_server) \
	       [ libdb get $myDb aaa:acct_profile:$acct_set:acct_server ]
      }
  }

  set sess_sets_in_db [ libdb iterate children $myDb aaa:sess_profile ]
  set sess_sets [list]
  foreach sess_set $sess_sets_in_db {
      set aaa_current(aaa:sess_profile:$sess_set) \
			 [ libdb get $myDb aaa:sess_profile:$sess_set ] 
      if {  $aaa_current(aaa:sess_profile:$sess_set) == "t"  } {
	lappend sess_sets $sess_set
	set aaa_current(aaa:sess_profile:$sess_set:sess_type) \
	       [ libdb get $myDb aaa:sess_profile:$sess_set:sess_type ]
	set aaa_current(aaa:sess_profile:$sess_set:sess_control_flag) \
	       [ libdb get $myDb aaa:sess_profile:$sess_set:sess_control_flag ]
      }
  }

  # init new: variables, if they don't exist
  # otherwise, they retain their current values
  if { [info exists aaa_current(new:aaa:service)] == 0 } {
      set aaa_current(new:aaa:service) ""
  }

  if { [info exists aaa_current(new:aaa:service:profile)] == 0 } {
      set aaa_current(new:aaa:service:profile) ""
  }

  if { [info exists aaa_current(new:aaa:profile)] == 0 } {
      set aaa_current(new:aaa:profile) ""
  }

  if { [info exists aaa_current(new:aaa:profile:auth_profile)] == 0 } {
      set aaa_current(new:aaa:profile:auth_profile) ""
  }

  if { [info exists aaa_current(new:aaa:profile:acct_profile)] == 0 } {
      set aaa_current(new:aaa:profile:acct_profile) ""
  }

  if { [info exists aaa_current(new:aaa:profile:sess_profile)] == 0 } {
      set aaa_current(new:aaa:profile:sess_profile) ""
  }

  if { [info exists aaa_current(new:aaa:auth_profile)] == 0 } {
      set aaa_current(new:aaa:auth_profile) ""
  }

  if { [info exists aaa_current(new:aaa:auth_profile:auth_type)] == 0 } {
      # set aaa_current(new:aaa:auth_profile:auth_type) [lindex $auth_pam_types 0]
      set aaa_current(new:aaa:auth_profile:auth_type) ""
  }

  if { [info exists aaa_current(new:aaa:auth_profile:auth_control_flag)] == 0 }\
  {
      set aaa_current(new:aaa:auth_profile:auth_control_flag) "required"
  }

  if { [info exists aaa_current(new:aaa:auth_profile:file_path)] == 0 }\
  {
      set aaa_current(new:aaa:auth_profile:file_path) ""
  }

  if { [info exists aaa_current(new:aaa:acct_profile)] == 0 } {
      set aaa_current(new:aaa:acct_profile) ""
  }

  if { [info exists aaa_current(new:aaa:acct_profile:acct_type)] == 0 } {
      # set aaa_current(new:aaa:acct_profile:acct_type) [lindex $acct_pam_types 0]
      set aaa_current(new:aaa:acct_profile:acct_type) ""
  }

  if { [info exists aaa_current(new:aaa:acct_profile:acct_control_flag)] == 0 } {
      set aaa_current(new:aaa:acct_profile:acct_control_flag) "required"
  }

  if { [info exists aaa_current(new:aaa:acct_profile:file_path)] == 0 } {
      set aaa_current(new:aaa:acct_profile:file_path) ""
  }

  if { [info exists aaa_current(new:aaa:sess_profile)] == 0 } {
      set aaa_current(new:aaa:sess_profile) ""
  }

  if { [info exists aaa_current(new:aaa:sess_profile:sess_type)] == 0 } {
      # set aaa_current(new:aaa:sess_profile:sess_type) [lindex $sess_pam_types 0]
      set aaa_current(new:aaa:sess_profile:sess_type) ""
  }

  if { [info exists aaa_current(new:aaa:sess_profile:sess_control_flag)] == 0 }\
  {
      set aaa_current(new:aaa:sess_profile:sess_control_flag) "required"
  }

  if { [info exists aaa_current(new:aaa:sess_profile:servers)] == 0 } {
      set aaa_current(new:aaa:sess_profile:servers) ""
  }

}

#********************************************************************
# Procedure   : restore_defaults
# Description :
# Arguments   :
# Returns     :
# See also    :
#*******************************************************************/

proc restore_defaults {} {
    global aaa_current
    global auth_pam_types acct_pam_types sess_pam_types

    set aaa_current(new:aaa:service) ""
    set aaa_current(new:aaa:service:profile) ""
    set aaa_current(new:aaa:profile) ""
    set aaa_current(new:aaa:profile:auth_profile) ""
    set aaa_current(new:aaa:profile:acct_profile) ""
    set aaa_current(new:aaa:profile:sess_profile) ""
    set aaa_current(new:aaa:auth_profile) ""
    # set aaa_current(new:aaa:auth_profile:auth_type) [lindex $auth_pam_types 0]
    set aaa_current(new:aaa:auth_profile:auth_type) ""
    set aaa_current(new:aaa:auth_profile:auth_control_flag) "required"
    set aaa_current(new:aaa:auth_profile:file_path) ""
    set aaa_current(new:aaa:acct_profile) ""
    # set aaa_current(new:aaa:acct_profile:acct_type) [lindex $acct_pam_types 0]
    set aaa_current(new:aaa:acct_profile:acct_type) ""
    set aaa_current(new:aaa:acct_profile:acct_control_flag) "required"
    set aaa_current(new:aaa:acct_profile:file_path) ""
    set aaa_current(new:aaa:sess_profile) ""
    # set aaa_current(new:aaa:sess_profile:sess_type) [lindex $sess_pam_types 0]
    set aaa_current(new:aaa:sess_profile:sess_type) ""
    set aaa_current(new:aaa:sess_profile:sess_control_flag) "required"
    set aaa_current(new:aaa:sess_profile:servers) ""
}

#********************************************************************
# Procedure   : pam_type_chgd
# Description :
# Arguments   :
# Returns     :
# See also    :
#*******************************************************************/

proc pam_type_chgd { binding value } {
    global auth_pam_types acct_pam_types sess_pam_types

    set params [split $binding :]
    set type [lindex $params end]
    switch -exact -- $type {
	auth_type { set types $auth_pam_types }
	acct_type { set types $acct_pam_types }
	sess_type { set types $sess_pam_types }
	default { return 0 }
    }
    if { $value == [lindex $types 0] } {
	return 0
    } else {
	return 1
    }
}

#********************************************************************
# Procedure   : more_profile_params
# Description :
# Arguments   :
# Returns     :
# See also    :
#*******************************************************************/

proc more_profile_params { binding new_params } {
    set params [split $binding :]
    set param [lindex $params 2]
    set found [lsearch -regexp $new_params "aaa:new:$param"]
    if { $found == -1 } {
	return 0
    } else {
	return 1
    }
}

#********************************************************************
# Procedure   : do_select_delete
# Description :
# Arguments   :
# Returns     :
# See also    :
#*******************************************************************/

proc do_select_delete { i var } {
    global Q_Names aaa_debug myDb err_list dbgf set_list

    if { $aaa_debug == 7 } {
        puts $dbgf "do_select_delete: $i, $var"
    }
    # processes "select:" into CDK deletes
    # scan Q_Names from entry "i" forward until the "delete:" entry is hit
    set sub_deletes 0
    while { $i != -1 } {
        set curr_var [lindex $Q_Names $i]
	if { [ regexp {^delete:} $curr_var ] == 1 } {
            set i -1
	    continue
	} 

        if { $aaa_debug == 7 } {
	    puts $dbgf "\tvar($i): $curr_var "
        }
        set curr_val [getVal $curr_var]
        # format of value is "<num>:<name>", where <num> is 0 for blank entries
        set vparts [split $curr_val :]
        if { [lindex $vparts 0] == 0 } {
            incr i
	    continue
	} 

        if { $aaa_debug == 7 } {
	    puts $dbgf "\tval(0): [lindex $vparts 0]"
	    puts $dbgf "\tval(1): [lindex $vparts 1]"
        }

	incr sub_deletes
	regexp "select:$var:(.*)" $curr_var match item
	set dbvar "$var:$item:[lindex $vparts 0]:name"

        if { $aaa_debug == 7 } {
	    puts $dbgf "\tvar deleted: $dbvar "
        }
	lappend set_list $dbvar ""
        incr i
    }

    if { $sub_deletes == 0 } {
        if { $aaa_debug == 7 } {
	        puts $dbgf "\tvar deleted: $var "
        }
	lappend set_list $var ""
    }
}

#********************************************************************
# Procedure   : cmp_ip_addr
# Description :
# Arguments   :
# Returns     :
# See also    :
#*******************************************************************/

proc cmp_ip_addr { a b } {
    set ew1 [split $a :]
    set ew2 [split $b :]
    set pool_name_ck [string compare -nocase [lindex $ew1 2] [lindex $ew2 2]]
    if { $pool_name_ck != 0 } {
	return $pool_name_ck
    } else {
	return [expr [lindex $ew1 end] - [lindex $ew2 end]]
    }
}

