#! /usr/bin/ipstcl

######################################################################
# Copyright 2000 Nokia IP, Inc.  All rights reserved.
#
# YOU MAY ONLY USE THE MATERIALS FOUND IN THIS FILE PURSUANT TO THE
# TERMS OF THE EVALUATION LICENSE AGREEMENT FOUND IN "/LICENSE".  ALL
# OTHER USES ARE EXPRESSLY PROHIBITED.
######################################################################
#
# $Id: aaa.tcl,v 1.35 2005/12/22 12:18:58 bangar Exp $
#

# CGI Tcl script for Authentication, Authorization and Accounting (AAA)
# feature
#

source check_cluster.tcl
ipso

# define subroutines
source aaa_subs.tcl
source validate.tcl
source show_result.tcl

# set Commit_button "<B>Commit</B>"
set Commit_button "<B>Apply</B>"
set set_list [list]
set err_list [list]

set is_cluster [check_cluster]
#cid will be -1 if is_cluster is false
set cid [get_cluster_id]

set UP "main"
set TOP main

if {[catch {libdb init -local -user $USER} myDb]} {
    # failed to initialize!!!
    lappend err_list "Unable to connect to database: ${myDb}"
    HTML_Start "HullPlated AAA Configuration" aaa.tcl
    show_result 1
    HTML_End
    set myDb {}
    exit 1
}

#initialize the cluster handle if we are running in CV
if {$is_cluster} {
   if {[catch {cluster init -cid $cid -user $USER} myCl]} {
      #failed to initialize!!!
      lappend err_list "Unable to connect to cluster: ${myCl}\n"
      set APPLY_REQ false
      set SAVE_REQ false
      HTML_Start "HullPlated Cluster AAA Configuration" aaa.tcl
      show_result 1
      HTML_End
      set myCl {}
      libdb close $myDb
      exit 1
   }
   print_feature_warning "aaa" $myDb $cid
} else {
    print_feature_warning "aaa" $myDb ""
}




set aaa_debug 0
set opresult 0

set method [get_Method] 

if { $aaa_debug > 0 } {
  set dbgf [open /tmp/aaadebug.txt w]
  puts $dbgf "aaa_debug is $aaa_debug"
}

# set types 
set pam_ctrl_types [list "required" "requisite" "sufficient" "optional"]
set pam_auth_ctrl_types [concat $pam_ctrl_types "nokia-server-auth-sufficient"]
# A list of combinations of AAA types (modules) and functions (auth/acct/sess)
# that have server configuration, which goes in a different script,
# linked from here.
set have_aaaservers {RADIUS auth RADIUS sess TACPLUS auth TACPLUS sess}

# set Profile Types from available PAM modules
# old: set pam_types [list "LOCAL" "RADIUS" "TACACS"]
# segregate by suffix: "auth", "acct" or "sess"
# modules with no suffix are placed in all three lists
# modules with a suffix other than "auth", "acct" or "sess" are not included
set module_list [ lsort [ glob -nocomplain -- /usr/lib/pam_*\.so\.* ] ]
if { $aaa_debug == 3 } {
  puts $dbgf "module list:"
  puts $dbgf "[ join $module_list "\n" ]"
}
set auth_pam_types [list]
set acct_pam_types [list]
set sess_pam_types [list]
foreach module $module_list {
  # skip Unit Test modules
  if { [regexp {test} $module] == 1 } { continue }

  regexp -nocase -- {/usr/lib/pam_(.+)\.so*} $module match modname
  set modparts [split $modname "_"]
  set modbase [lindex $modparts 0]
  set modext [lindex $modparts 1]

  if { $aaa_debug == 5 } {
    puts $dbgf "module proc: match=$match, base=$modbase, ext=$modext"
  }

  # disable special handling for TACACS module
  # -- remove '# ' from next 3 lines to enable
  # if { $modbase == "tacplus" } {
  #   set modbase tacacs
  # } 
  if { [string length $modext] == 0 || [string compare $modext "auth"] == 0 } {
    set pam_type [string toupper $modbase]
    # skip if module is already in list
    if { [lsearch -exact $auth_pam_types $pam_type] > 0 } { continue }
    if { $aaa_debug == 3 } {
      puts $dbgf "adding \"$pam_type\" to Auth. Profile Types"
    }
    lappend auth_pam_types $pam_type
  }
  if { [string length $modext] == 0 || [string compare $modext "acct"] == 0 } {
    set pam_type [string toupper $modbase]
    # skip if module is already in list
    if { [lsearch -exact $acct_pam_types $pam_type] > 0 } { continue }
    if { $aaa_debug == 3 } {
      puts $dbgf "adding \"$pam_type\" to Acct. Profile Types"
    }
    lappend acct_pam_types $pam_type
  }
  if { [string length $modext] == 0 || [string compare $modext "sess"] == 0 } {
    set pam_type [string toupper $modbase]
    # skip if module is already in list
    if { [lsearch -exact $sess_pam_types $pam_type] > 0 } { continue }
    if { $aaa_debug == 3 } {
      puts $dbgf "adding \"$pam_type\" to Session Profile Types"
    }
    lappend sess_pam_types $pam_type
  }
}

acquire_aaa_config

if { $method == "POST" } { 
    if { $apply == 1 } {
      # process results of Commit here 
      if { $aaa_debug == 2 } {
        puts $dbgf "aaa_current in main:"
        puts $dbgf "[ array names aaa_current ]"
      }

      set opresult [check_aaa_bindings]

      if { $opresult == 0 } {
        if {$save == 1} {
          if {$is_cluster} {
            if {[catch {cluster db set $myCl :save} set_result]} { 
              lappend err_list  $set_result
              set set_result {}
              set opresult 1
            }
  	  } elseif {[catch {libdb set $myDb :save} set_result]} {
  	    lappend err_list  $set_result
  	    set set_result {}
  	    set opresult 1
          }
        }
      }
    }

    if { [llength $err_list] == 0 } {
	restore_defaults
    }

} elseif { $method == "GET" } { 
    # your processing code goes here 
} else {
    puts "No CLI support at this time!"
    exit 0
}

# the "config_top" replacement included here automatically
set docdir [libdb get $myDb "docdir"]
set DOC "${docdir}/security.htm"
if {$is_cluster} {
    HTML_Start "HullPlated Cluster AAA Configuration" aaa.tcl
} else {
    HTML_Start "HullPlated AAA Configuration" aaa.tcl
}
show_result $opresult

if { $aaa_debug == 1 && $method == "POST" } {
    HTML_Table "Posted Bindings" "Name" "Value"
    foreach var $Q_Names {
        puts  $dbgf "$var: [getVal $var]"
        HTML_Col
        TEXT  "$var"
        HTML_Col
        TEXT "[getVal $var]"
        HTML_Row
    }
    HTML_EndTable
    HTML_HR
}

HELP "Multiple alterations and/or additions can be made to items \
	in the tables \
	below prior to selecting the $Commit_button button.\
	<BR>\
	<BR>\
	Names are composed of alphanumeric characters \
	and underbar (\\'_\\'). \
	It is <B>strongly</B> recommended that a \
	naming convention be adopted. \
	<BR>\
	<BR>\
	In general, Services or Profiles cannot be deleted \
	if they are referenced by other definitions; e.g., \
	when the <B>Service </B>\\'my_service\\' has \
	<B>Profile </B>\\'my_profile\\', \\'my_profile\\' cannot be \
	deleted until \\'my_service\\' is deleted.  Both a Service \
	and any or all of its Profiles can be deleted at the same time. "

#   in this table, cells have the following controls when they are present:
#     column 1: string = service name
#     column 2: edit box = service profile name
#     column 3: check box = unchecked

HTML_GroupStart "Service Module Configuration"
if { [ llength $svc_names ] > 0 } {
  HTML_Table "Service Module" "Service" "Service Profile" "Delete"
  # place the rows of the table  here
  set svc_rowi [ llength $svc_names ]
  set i 0
  foreach svc_name $svc_names {
      HTML_Col
        TEXT $svc_name 
      HTML_Col
        get_String "" aaa:service:$svc_name:profile \
	            $aaa_current(aaa:service:$svc_name:profile) ""
      HTML_Col CENTERALIGN
        HTML_CheckBox "" delete:aaa:service:$svc_name:profile ""
      HTML_Row
      incr i
  }
  HTML_Row
  HTML_Col
    get_String ""  new:aaa:service $aaa_current(new:aaa:service) 1
  HTML_Col
    # get_String ""  new:aaa:service:profile $aaa_current(new:aaa:service:profile) 1
    HTML_List "" new:aaa:service:profile 1
    HTML_Item "" ""
    foreach prof_set $svc_profiles {
      HTML_Item $prof_set $prof_set
    }
    HTML_EndList


  HTML_EndTable

  HELP "<B>Service Module Configuration</B>\
	  <UL>\
          <LI><B>Service</B> is the name of an application or service that is \
                  to use AAA.\
          <LI><B>Profile</B> is an entry in the <B>Service Profile</B> table. \
          <LI><B>Delete</B> deletes the Service/Profile association. \
	  </UL>\
       To alter the Service/Profile association, type the name of the new \
       Service Profile into the edit box under the column labeled \
       <B>Profile</B>, followed by the $Commit_button button.\
	<BR>\
	<BR>\
       To delete a Service/Profile association, select the check box \
       in the <B>Delete</B> column, \
        followed by the $Commit_button button.\
	<BR>\
	<BR>\
       <B>Warning</B>: \
       the following Service/Profile associations cannot be \
       deleted:\
       <UL>\
       <LI><TT>httpd</TT> \
       <LI><TT>login</TT> \
       <LI><TT>other</TT> \
       <LI><TT>snmpd</TT> \
       <LI><TT>sshd</TT> \
       </UL>"
}

HELP "Use these controls to create a new Service/Profile association.\
	  <UL>\
          <LI>Set <B>New Service</B> to the name of an application or service \
            not in the table.\
          <LI>Set <B>Profile</B> to the Service Profile for the application or \
             service. If the name is not in the <B>Service Profile  </B> \
	     table, it must also be added below. \
	  </UL>\
      Use the $Commit_button button to create the new \
      Service/Profile association. It appears in the table when the page is \
      displayed. "
HTML_BR
TEXT "You should never need to create a new Service Module entry other than the standard httpd, login, snmpd, sshd and other entries."
HTML_GroupEnd
HTML_BR

#   in this table, cells have the following controls when they are present:
#     column 1: string = service profile name
#     column 2: list box = auth. profile names
#     column 3: list box = acct. profile names
#     column 4: list box = sess. profile names
#     column 5: check box = unchecked

HTML_BR 
HTML_GroupStart "Service Profile Configuration"
set blankname "        "
set prof_sets [ llength $svc_profiles ]
if { $prof_sets > 0 } {
    HTML_Table "Service Profile" "Profile Name" "Auth Profile" \
	           "Acct Profile" "Session Profile" "Delete"
    # place the rows of the table  here
    foreach prof_set $svc_profiles {
      set prof_auth_profiles [list 0 $blankname]
      set prof_acct_profiles [list 0 $blankname]
      set prof_sess_profiles [list 0 $blankname]
      set svc_authcnt [libdb get $myDb aaa:profile:$prof_set:num_auth_modules]
      if { $svc_authcnt == "" } {
        set svc_authcnt 1
      } else {
        set i 1
        while { $i <= $svc_authcnt } {
          lappend prof_auth_profiles $i\
           [ libdb get $myDb aaa:profile:$prof_set:auth_profile:$i:name ]
          incr i
        }
	incr svc_authcnt
      }
      set svc_acctcnt [libdb get $myDb aaa:profile:$prof_set:num_acct_modules]
      if { $svc_acctcnt == "" } {
        set svc_acctcnt 1
      } else {
        set i 1
        while { $i <= $svc_acctcnt } {
          lappend prof_acct_profiles $i\
             [ libdb get $myDb aaa:profile:$prof_set:acct_profile:$i:name ]
          incr i
        }
	incr svc_acctcnt
      }
      set svc_sesscnt [libdb get $myDb aaa:profile:$prof_set:num_sess_modules]
      if { $svc_sesscnt == "" } {
        set svc_sesscnt 1
      } else {
        set i 1
        while { $i <= $svc_sesscnt } {
          lappend prof_sess_profiles $i\
           [ libdb get $myDb aaa:profile:$prof_set:sess_profile:$i:name ]
          incr i
        }
	incr svc_sesscnt
      }
      HTML_Col
        TEXT $prof_set
      HTML_Col
	# HTML_List "" select:aaa:profile:$prof_set:auth_profile $svc_authcnt
        HTML_List "" select:aaa:profile:$prof_set:auth_profile 4

          set i 0
          while { $i < [llength $prof_auth_profiles] } {
            set item_index [ lindex $prof_auth_profiles $i ]
            set auth_val [ lindex $prof_auth_profiles [expr $i + 1] ]
            HTML_Item $auth_val $item_index:$auth_val
            incr i 2
	  }
        HTML_EndList
      HTML_Col
        HTML_List "" select:aaa:profile:$prof_set:acct_profile 4
 # $svc_acctcnt
          set i 0
          while { $i < [llength $prof_acct_profiles] } {
            set item_index [ lindex $prof_acct_profiles $i ]
            set acct_val [ lindex $prof_acct_profiles [expr $i + 1] ]
            HTML_Item $acct_val $item_index:$acct_val
            incr i 2
	  }
        HTML_EndList
      HTML_Col
        HTML_List "" select:aaa:profile:$prof_set:sess_profile 4
#$svc_sesscnt
          set i 0
          while { $i < [llength $prof_sess_profiles] } {
            set item_index [ lindex $prof_sess_profiles $i ]
            set sess_val [ lindex $prof_sess_profiles [expr $i + 1] ]
            HTML_Item $sess_val $item_index:$sess_val
            incr i 2
	  }
        HTML_EndList
      HTML_Col CENTERALIGN
        HTML_CheckBox "" delete:aaa:profile:$prof_set ""
      HTML_Row
    }
    HTML_EndTable

    HELP "<B>Service Profile</B>\
	  <UL>\
	  <LI><B>Profile Name</B> is the name of the Profile. \
	    This name can be used in the \
            <B>Profile </B>column of the <B>Service Module Configuration \
	    </B>table.\
	  <LI><B>Auth. Profile</B> is a list of entries from the <B>Auth. \
	    Profile </B>table.\
	  <LI><B>Acct. Profile</B> is a list of entries from the <B>Acct. \
	    Profile </B>table.\
	  <LI><B>Session Profile</B> is a list of entries from the \
	    <B>Session Profile </B>table.\
          <LI><B>Delete</B> deletes the Profile. \
	  </UL>\
       To delete a Service Profile, select the check box \
       in the <B>Delete</B> column, \
       followed by the $Commit_button button. \
       <BR>\
       <BR>\
       To delete an item in the <B>Auth. Profile</B>, <B>Acct. Profile \
       </B>and/or <B>Session Profile </B>lists, select the item and
       the check box in the <B>Delete</B> column, followed by the \
       $Commit_button button. "
}

HTML_BR
HTML_Table "" "" noborder
HTML_Row
HTML_Col
HTML_Label "Service Profile"
HTML_Col
get_String "" new:aaa:profile \
           $aaa_current(new:aaa:profile) 1
HTML_Col
HTML_Label "Auth. Profile"
HTML_Col
get_String "" new:aaa:profile:auth_profile \
           $aaa_current(new:aaa:profile:auth_profile) 1
HTML_Row
HTML_Col
HTML_Label "Acct. Profile"
HTML_Col
get_String "" new:aaa:profile:acct_profile \
           $aaa_current(new:aaa:profile:acct_profile) 1
HTML_Col
HTML_Label "Session Profile"
HTML_Col
get_String "" new:aaa:profile:sess_profile \
           $aaa_current(new:aaa:profile:sess_profile) 1
HTML_EndTable

HELP "Use these controls to create a new Service Profile or add a new \
      Auth. Profile, Acct. Profile and/or Session Profile to an existing \
      Service Profile.\
     <UL>\
     <LI>Set <B>Service Profile</B> to either the name of a Service Profile \
       in the table or the name of a new Service Profile to add to the table: \
       <UL>\
       <LI>When the name is one of those in the table, the values entered \
         into the Auth. Profile, Acct. Profile and/or Session Profile \
	 edit boxes are added to the lists. <BR>\
       <LI>When the the name is not in the table, a new Service Profile \
         is added with the values for Auth. Profile, Acct. Profile \
         and/or Session Profile specified.\
      </UL>\
     <LI>Set <B>Auth. Profile</B> to the Auth. Profile. \
       If the name is not in the <B>Auth. Profile  </B> \
	     table, it must also be added below. \
     <LI>Set <B>Acct. Profile</B> to the Acct. Profile. \
        If the name is not in the <B>Acct. Profile  </B> \
	     table, it must also be added below. \
     <LI>Set <B>Session Profile</B> to the Session Profile. \
       If the name is not in the <B>Session Profile  </B> \
	     table, it must also be added below. \
     </UL>\
      <B>Note: </B>At least one Auth. Profile, Acct. Profile\
      or Session Profile must be specified. \
      <BR> \
      <BR> \
     Use the $Commit_button button to create the \
      new Service Profile or add the new Auth. Profiles, Acct. Profiles and/or \
      Session Profiles to an existing Service Profile. The changes appear in \
      the table when the page is displayed.  "

# future features for Auth., Acct. and Session Profiles (if needed):
#  
#      multiple entries can be entered, separated by commas or spaces.
#

HTML_GroupEnd
HTML_BR
HTML_BR

#   in this table, cells have the following controls when they are present:
#     column 1: string = auth. profile name
#     column 2: selection box = UNIX/SKEY/ROOTOK/...
#     column 3: selection box = required/requisite/sufficient/optional
#     column 4: edit box = server_name/file_name
#     column 5: check box = unchecked

HTML_GroupStart "Authentication, Accounting and Session Management Objects"
if { [llength $auth_sets] > 0 } {
  HTML_Table "Auth. Profile" "Name" "Type" "Control" "File" \
             "Delete" "Server Link"
    foreach auth_set $auth_sets {
      HTML_Col
        TEXT $auth_set
      HTML_Col
	HTML_List "" aaa:auth_profile:$auth_set:auth_type 1
	  set type $aaa_current(aaa:auth_profile:$auth_set:auth_type)
          foreach pam_type $auth_pam_types {
            if { [string compare $pam_type $type] == 0 } {
              HTML_Item $pam_type $pam_type 1
            } else {
              HTML_Item $pam_type $pam_type
            }
          }
        HTML_EndList
      HTML_Col
        HTML_List "" aaa:auth_profile:$auth_set:auth_control_flag 1
	  set cntl_flag \
	         $aaa_current(aaa:auth_profile:$auth_set:auth_control_flag)
          foreach ctrl_type $pam_auth_ctrl_types {
            if { [string compare $ctrl_type $cntl_flag] == 0 } {
              HTML_Item $ctrl_type $ctrl_type 1
            } else {
              HTML_Item $ctrl_type $ctrl_type
            }
          }
        HTML_EndList
      HTML_Col
	if { $aaa_current(aaa:auth_profile:$auth_set:auth_type) == "LOCAL" } {
          get_String "" aaa:auth_profile:$auth_set:file_path \
	           $aaa_current(aaa:auth_profile:$auth_set:file_path) \
		   1
	}
      HTML_Col CENTERALIGN
        HTML_CheckBox "" delete:aaa:auth_profile:$auth_set ""
	# Might this have "server" configuration?  If so, link to it.
        HTML_Col CENTERALIGN
	foreach {hs_type hs_func} $have_aaaservers {
	    TEXT " "
	    if {$hs_func == "auth"} {
		if {$hs_type == $type} {
		    set link "aaa_servers.tcl"
		    set link "${link}?function=auth"
		    set link "${link}&module=${hs_type}"
		    set link "${link}&profile=${auth_set}"
		    HTML_Link $link "Servers"
		    break
		} 
	    }
	}
      HTML_Row
    }
    HTML_Col
    get_String ""  new:aaa:auth_profile $aaa_current(new:aaa:auth_profile) 1
    HTML_Col
    HTML_List ""  new:aaa:auth_profile:auth_type 1
      foreach pam_type $auth_pam_types {
        if { $pam_type == $aaa_current(new:aaa:auth_profile:auth_type) } {
          HTML_Item $pam_type $pam_type 1
        } else {
          HTML_Item $pam_type $pam_type
        }
      }
    HTML_EndList
    HTML_Col
    HTML_List ""  new:aaa:auth_profile:auth_control_flag 1
      foreach ctrl_type $pam_auth_ctrl_types {
        if { $ctrl_type == $aaa_current(new:aaa:auth_profile:auth_control_flag) } {
          HTML_Item $ctrl_type $ctrl_type 1
        } else {
          HTML_Item $ctrl_type $ctrl_type
        }
      }
    HTML_EndList
    HTML_Col
  get_String ""  new:aaa:auth_profile:file_path $aaa_current(new:aaa:auth_profile:file_path) 1
  HTML_Col
  HTML_Col

HTML_Row
  HTML_EndTable

  HELP "<B>Auth. Profile</B>\
       <UL>\
       <LI><B>Name</B> is the name of the Auth. Profile.\
       <LI><B>Type</B> is the type of authentication to be performed.\
         &#40;For more information about what authentication algorithms are \
	 implemented by the items in the list, see the on-line help.&#41; \
       <LI><B>Control</B> determines how the result of authentication is to \
         be applied; for an explanation of the selections, see the \
         description under <B>Control Items</B>.\
       <LI><B>File</B> is unused at this time. \
       <LI><B>Delete</B> deletes the Auth. Profile. \
       <LI><I>Note:</I> Be careful when configuring the &quot;RADIUS&quot; \
	and &quot;TACPLUS&quot; \
	auth. modules with the &quot;httpd&quot; service.  The way they work \
	might not be quite what you&#39;re expecting.  See the release notes \
	and on-line documentation for more details. \
       </UL>\
       To alter <B>Type</B> or <B>Control</B>, select a different item on \
       the selection control under the appropriate column, followed by \
       the $Commit_button button.\
        <BR>\
        <BR>\
       To delete an Auth. Profile, select the check box \
       in the <B>Delete</B> column, \
        followed by the $Commit_button button."
}

# use table control to force selection lists onto same line

HELP "Use these controls to create a new Auth. Profile.\
    <UL>\
    <LI>Set <B>New Auth. Profile</B> to a name of an Auth. Profile not in the \
      table.\
    <LI>Set <B>Type</B> to the type of authentication for the new \
      Auth. Profile.\
      &#40;For more information about what authentication algorithms are \
      implemented by the items in the list, see the on-line help.&#41; \
    <LI>Set <B>Control</B> to the type of control for the new \
      Auth. Profile; for an explanation of the selections, see the \
      description under <B>Control Items</B>.\
    <LI><B>File</B> is unused at this time; leave it blank.\
       <LI><I>Note:</I> Be careful when configuring the &quot;RADIUS&quot; \
	and &quot;TACPLUS&quot; \
	auth. modules with the &quot;httpd&quot; service.  The way they work \
	might not be quite what you&#39;re expecting.  See the release notes \
	and on-line documentation for more details. \
    </UL>\
    Use the $Commit_button button to create the new \
    Auth. Profile. It appears in the table when the page is displayed. "


HELP "<LI><B>Control Items</B> \
      <BR>\
       The value of the <B>Control</B> selection list \
       determines how the result of the Pluggable Module associated with \
       the <B>Type </B> of an Auth. Profile, Acct. Profile or \
       Session Profile is to be applied and when additional modules \
       in a list are invoked. The items have the following meanings:\
       <UL>\
       <LI><B>required</B> <BR> \
         the result is retained and the next module is invoked. \
       <LI><B>requisite</B> <BR> \
         a result of \\'error\\' is reported immediately \
	 and no further modules are invoked; \
	 <BR> \
	 otherwise, the result is retained and the next module is invoked.\
       <LI><B>sufficient</B> <BR> \
         if no previous module has reported \\'error\\', \
	 a result of \\'no error\\' is reported immediately and no further \
	 modules are invoked; \
	 <BR> \
	 a result of \\'error\\' for this module is discarded; \
	 <BR> \
	 if a previous module has reported \\'error\\' or \
	 the result of this module is \\'error\\', the next module \
	 is invoked.\
       <LI><B>optional</B> <BR> \
         a result of \\'error\\' is ignored (discarded) and a result of \
	 \\'no error\\' is retained; \
	 <BR> \
	 the next module is always invoked.\
       <LI><B>nokia-server-auth-sufficient</B> (Auth. Profile only)<BR> \
	 identical to <B>sufficient </B>, except a result of \
	 \\'authentication error\\' for this module is reported \
	 immediately and no further modules are invoked; \
	 (<B>Note: </B>this selection is intended for use \
	 with modules that access remote servers and where the modules \
	 have different result codes for \\'authentication error\\' \
	 and \\'server not reachable\\'.) \
       </UL> \
       A value other than <B>required </B>is only effective \
       when multiple items are defined in the <B>Auth. Profile</B>, \
       <B>Acct. Profile </B>or <B>Session Profile </B>lists for a given \
       <B>Service Profile</B>, a feature known as &quot;stacking&quot;."

HTML_BR

#   in this table, cells have the following controls when they are present:
#     column 1: string = acct. profile name
#     column 2: selection box = UNIX/PERMIT/...
#     column 3: selection box = required/requisite/sufficient/optional
#     column 4: edit box = server_name/file_name
#     column 5: check box = unchecked

if { [llength $acct_sets] > 0 } {
  HTML_Table "Acct. Profile" "Name" "Type" "Control" "File" \
             "Delete" "Server Link"
    foreach acct_set $acct_sets {
      HTML_Col
        TEXT $acct_set
      HTML_Col
	HTML_List "" aaa:acct_profile:$acct_set:acct_type 1
          set type $aaa_current(aaa:acct_profile:$acct_set:acct_type)
          foreach pam_type $acct_pam_types {
            if { [string compare $pam_type $type] == 0 } {
              HTML_Item $pam_type $pam_type 1
            } else {
              HTML_Item $pam_type $pam_type
            }
          }
        HTML_EndList
      HTML_Col
        HTML_List "" aaa:acct_profile:$acct_set:acct_control_flag 1
          set cntl_flag \
	       $aaa_current(aaa:acct_profile:$acct_set:acct_control_flag)
          foreach ctrl_type $pam_ctrl_types {
            if { [string compare $ctrl_type $cntl_flag] == 0 } {
              HTML_Item $ctrl_type $ctrl_type 1
            } else {
              HTML_Item $ctrl_type $ctrl_type
            }
          }
        HTML_EndList
      HTML_Col
        if { $aaa_current(aaa:acct_profile:$acct_set:acct_type) == "LOCAL" } {
          get_String "" aaa:acct_profile:$acct_set:file_path \
	           $aaa_current(aaa:acct_profile:$acct_set:file_path) \
		   1
	}
      HTML_Col CENTERALIGN
        HTML_CheckBox "" delete:aaa:acct_profile:$acct_set ""
	# Might this have "server" configuration?  If so, link to it.
        HTML_Col CENTERALIGN
	foreach {hs_type hs_func} $have_aaaservers {
	    TEXT " "
	    if {$hs_func == "acct"} {
		if {$hs_type == $type} {
		    set link "aaa_servers.tcl"
		    set link "${link}?function=acct"
		    set link "${link}&module=${hs_type}"
		    set link "${link}&profile=${acct_set}"
		    HTML_Link $link "Servers"
		    break
		}
	    }
	}
      HTML_Row
    }

  HTML_Col
    get_String ""  new:aaa:acct_profile \
               $aaa_current(new:aaa:acct_profile) 1
  HTML_Col
    HTML_List ""  new:aaa:acct_profile:acct_type 1
      foreach pam_type $acct_pam_types {
        if { $pam_type == $aaa_current(new:aaa:acct_profile:acct_type) } {
          HTML_Item $pam_type $pam_type 1
        } else {
          HTML_Item $pam_type $pam_type
        }
      }
    HTML_EndList
  HTML_Col
    HTML_List ""  new:aaa:acct_profile:acct_control_flag 1
      foreach ctrl_type $pam_ctrl_types {
        if { $ctrl_type == \
             $aaa_current(new:aaa:acct_profile:acct_control_flag) } {
          HTML_Item $ctrl_type $ctrl_type 1
        } else {
          HTML_Item $ctrl_type $ctrl_type
        }
      }
    HTML_EndList
  HTML_Col
  get_String ""  new:aaa:acct_profile:file_path \
           $aaa_current(new:aaa:acct_profile:file_path) 1
  HTML_Col
  HTML_Col
  HTML_Row

  HTML_EndTable
  HTML_BR

  HELP "<B>Acct. Profile</B>\
       <UL>\
       <LI><B>Name</B> is the name of Account Management Profile.\
       <LI><B>Type</B> is the type of account management to be performed.\
         &#40;For more information about what account management algorithms \
         are implemented by the items in the list, see the on-line help.&#41; \
       <LI><B>Control</B> determines how the result of account management \
         is to be applied; for an explanation of the selections, see the \
         description under <B>Control Items</B>.\
       <LI><B>File</B> is unused at this time. \
       <LI><B>Delete</B> deletes the Acct. Profile. \
       </UL>\
       To alter <B>Type</B> or <B>Control</B>, select a different item on \
       the selection control under the appropriate column, followed by \
       the $Commit_button button.\
        <BR>\
        <BR>\
       To alter <B>File</B>, type the name of the new file \
        into the edit box under the column labelled <B>File</B>, \
        followed by the $Commit_button button. \
	<BR>\
	<BR>\
       To delete an Acct. Profile, select the check box \
       in the <B>Delete</B> column, \
        followed by the $Commit_button button."
}

HELP "Use these controls to create a new Acct. Profile.\
    <UL>\
    <LI>Set <B>New Acct. Profile</B> to a name of an Acct. Profile not in the \
      table.\
    <LI>Set <B>Type</B> to the type of account management for the new Acct. \
      Profile.\
      &#40;For more information about what account management algorithms \
      are implemented by the items in the list, see the on-line help.&#41; \
    <LI>Set <B>Control</B> to the type of control for \
      the new Acct. Profile; for an explanation of the selections, see the \
      description under <B>Control Items</B>.\
    <LI><B>File</B> is unused at this time; leave it blank.\
    </UL>\
    Use the $Commit_button button to create the new \
    Acct. Profile. It appears in the table when the page is displayed. "

HTML_BR

#   in this table, cells have the following controls when they are present:
#     column 1: string = session profile name
#     column 2: selection box = UNIX/PERMIT/...
#     column 3: selection box = required/requisite/sufficient/optional
#     column 4: check box = unchecked

if { [llength $sess_sets] > 0 } {
  HTML_Table "Session Profile" "Name" "Type" "Control" "Delete"
    foreach sess_set $sess_sets {
      HTML_Col
        TEXT $sess_set
      HTML_Col
	HTML_List "" aaa:sess_profile:$sess_set:sess_type 1
          set type $aaa_current(aaa:sess_profile:$sess_set:sess_type)
          foreach pam_type $sess_pam_types {
            if { [string compare $pam_type $type] == 0 } {
              HTML_Item $pam_type $pam_type 1
            } else {
              HTML_Item $pam_type $pam_type
            }
          }
        HTML_EndList
      HTML_Col
        HTML_List "" aaa:sess_profile:$sess_set:sess_control_flag 1
          set cntl_flag \
	       $aaa_current(aaa:sess_profile:$sess_set:sess_control_flag)
          foreach ctrl_type $pam_ctrl_types {
            if { [string compare $ctrl_type $cntl_flag] == 0 } {
              HTML_Item $ctrl_type $ctrl_type 1
            } else {
              HTML_Item $ctrl_type $ctrl_type
            }
          }
        HTML_EndList
      HTML_Col CENTERALIGN
        HTML_CheckBox "" delete:aaa:sess_profile:$sess_set ""
	# Might this have "server" configuration?  If so, link to it.
	foreach {hs_type hs_func} $have_aaaservers {
	    if {$hs_func == "sess"} {
		if {$hs_type == $type} {
		    HTML_Col
		    set link "aaa_servers.tcl"
		    set link "${link}?function=sess"
		    set link "${link}&module=${hs_type}"
		    set link "${link}&profile=${sess_set}"
		    HTML_Link $link "Servers"
		    break
		}
	    }
	}
      HTML_Row
    }

  HTML_Col
    get_String ""  new:aaa:sess_profile \
               $aaa_current(new:aaa:sess_profile) 1
  HTML_Col
    HTML_List ""  new:aaa:sess_profile:sess_type 1
      foreach pam_type $sess_pam_types {
        if { $pam_type == \
             $aaa_current(new:aaa:sess_profile:sess_type) } {
          HTML_Item $pam_type $pam_type 1
        } else {
          HTML_Item $pam_type $pam_type
        }
      }
      HTML_EndList
  HTML_Col
    HTML_List ""  new:aaa:sess_profile:sess_control_flag 1
      foreach ctrl_type $pam_ctrl_types {
        if { $ctrl_type == \
             $aaa_current(new:aaa:sess_profile:sess_control_flag) } {
          HTML_Item $ctrl_type $ctrl_type 1
        } else {
          HTML_Item $ctrl_type $ctrl_type
        }
      }
      HTML_EndList
  HTML_Row

  HTML_EndTable

  HELP "<B>Session Profile</B>\
       <UL>\
       <LI><B>Name</B> is the name of Session Profile.\
       <LI><B>Type</B> is the type of session management to be performed.\
         &#40;For more information about what session management algorithms \
	 are implemented by the items in the list, see the on-line help.&#41; \
       <LI><B>Control</B> determines how the result of session management is \
         is to be applied; for an explanation of the selections, see the \
         description under <B>Control Items</B>.\
       <LI><B>Delete</B> deletes the Session Profile. \
       </UL>\
       To alter <B>Type</B> or <B>Control</B>, select a different item on \
       the selection control under the appropriate column, followed by \
       the $Commit_button button.\
        <BR>\
	<BR>\
       To delete a Session Profile, select the check box \
       in the <B>Delete</B> column, \
        followed by the $Commit_button button."
}

HTML_BR

HELP "Use these controls to create a new Session Profile.\
      <UL>\
      <LI>Set <B>New Session Profile</B> to a name of a Session Profile not \
        in the table.\
      <LI>Set <B>Type</B> to the type of session management for the new \
        Profile.\
        &#40;For more information about what session management algorithms \
	are implemented by the items in the list, see the on-line help.&#41; \
      <LI>Set <B>Control</B> to the type of control for the \
        new Profile; for an explanation of the selections, see the \
        description under <B>Control Items</B>.\
      </UL>\
     Use the $Commit_button button to create the new \
     Session Profile. It appears in the table when the page is displayed. "
HTML_GroupEnd

if { $aaa_debug > 0 } {
  flush $dbgf
  close $dbgf
}

libdb close $myDb

# the "config_bottom" replacement included here automatically
HTML_End

exit

